@@ -82,7 +82,7 @@ class ApproximationScheme(object):
 
         return self._approx_groups, self._colored_approx_groups
 
-    def add_approximation(self, abs_key, kwargs):
+    def add_approximation(self, abs_key, system, kwargs):
         """
         Use this approximation scheme to approximate the derivative d(of)/d(wrt).
 
@@ -90,6 +90,8 @@ class ApproximationScheme(object):
         ----------
         abs_key : tuple(str,str)
             Absolute name pairing of (of, wrt) for the derivative.
+        system : System
+            Containing System.
         kwargs : dict
             Additional keyword arguments, to be interpreted by sub-classes.
         """
@@ -137,18 +139,18 @@ class ApproximationScheme(object):
 
         is_total = isinstance(system, Group)
 
-        system._update_wrt_matches()
+        system._update_wrt_matches(system._coloring_info)
         wrt_matches = system._coloring_info['wrt_matches']
 
         data = None
         keys = set()
         for key, apprx in iteritems(self._exec_dict):
             if key[0] in wrt_matches:
+                if data is None:
+                    # data is the same for all colored approxs so we only need the first
+                    data = self._get_approx_data(system, key)
                 options = apprx[0][1]
                 if 'coloring' in options:
-                    if data is None:
-                        # data is the same for all colored approxs so we only need the first
-                        data = self._get_approx_data(system, key)
                     keys.update(a[0] for a in apprx)
 
         if is_total and system.pathname == '':  # top level approx totals
