@@ -118,8 +118,8 @@ class MetaModelUnStructuredComp(ExplicitComponent):
 
         if vec_size > 1:
             if metadata['shape'][0] != vec_size:
-                raise RuntimeError("Metamodel: First dimension of input '%s' must be %d"
-                                   % (name, vec_size))
+                raise RuntimeError("%s: First dimension of input '%s' must be %d"
+                                   % (self.msginfo, name, vec_size))
             input_size = metadata['value'][0].size
         else:
             input_size = metadata['value'].size
@@ -167,8 +167,8 @@ class MetaModelUnStructuredComp(ExplicitComponent):
 
         if vec_size > 1:
             if metadata['shape'][0] != vec_size:
-                raise RuntimeError("Metamodel: First dimension of output '%s' must be %d"
-                                   % (name, vec_size))
+                raise RuntimeError("%s: First dimension of output '%s' must be %d"
+                                   % (self.msginfo, name, vec_size))
             output_shape = metadata['shape'][1:]
             if len(output_shape) == 0:
                 output_shape = 1
@@ -559,15 +559,15 @@ class MetaModelUnStructuredComp(ExplicitComponent):
             if num_sample is None:
                 num_sample = len(val)
             elif len(val) != num_sample:
-                msg = "MetaModelUnStructuredComp: Each variable must have the same number"\
-                      " of training points. Expected {0} but found {1} "\
-                      "points for '{2}'."\
-                      .format(num_sample, len(val), name)
+                msg = "{}: Each variable must have the same number"\
+                      " of training points. Expected {} but found {} "\
+                      "points for '{}'."\
+                      .format(self.msginfo, num_sample, len(val), name)
                 raise RuntimeError(msg)
 
         if len(missing_training_data) > 0:
-            msg = "MetaModelUnStructuredComp: The following training data sets must be " \
-                  "provided as options for %s: " % self.pathname + \
+            msg = "%s: The following training data sets must be " \
+                  "provided as options: " % self.msginfo + \
                   str(missing_training_data)
             raise RuntimeError(msg)
 
@@ -604,8 +604,8 @@ class MetaModelUnStructuredComp(ExplicitComponent):
 
             surrogate = self._metadata(name).get('surrogate')
             if surrogate is None:
-                raise RuntimeError("Metamodel '%s': No surrogate specified for output '%s'"
-                                   % (self.pathname, name))
+                raise RuntimeError("%s: No surrogate specified for output '%s'"
+                                   % (self.msginfo, name))
             else:
                 surrogate.train(self._training_input,
                                 self._training_output[name])
