@@ -15,8 +15,8 @@ import numpy as np
 from scipy.sparse import coo_matrix
 
 from openmdao.api import Problem, Group, IndepVarComp, ImplicitComponent, ExecComp, \
-    ExplicitComponent, NonlinearBlockGS
-from openmdao.utils.assert_utils import assert_rel_error
+    ExplicitComponent, NonlinearBlockGS, ScipyOptimizeDriver
+from openmdao.utils.assert_utils import assert_rel_error, assert_warning
 from openmdao.utils.array_utils import evenly_distrib_idxs
 from openmdao.utils.mpi import MPI
 from openmdao.utils.coloring import compute_total_coloring
@@ -43,6 +43,8 @@ except ImportError:
 OPT, OPTIMIZER = set_pyoptsparse_opt('SLSQP')
 if OPTIMIZER:
     from openmdao.drivers.pyoptsparse_driver import pyOptSparseDriver
+else:
+    pyOptSparseDriver = None
 
 
 def setup_vars(self, ofs, wrts):
@@ -592,6 +594,39 @@ class TestColoring(unittest.TestCase):
         jac = comp._jacobian._subjacs_info
         _check_partial_matrix(comp, jac, sparsity, method)
 
+    def test_partials_min_improvement(self):
+        prob = Problem(coloring_dir=self.tempdir)
+        model = prob.model
+
+        mask = np.array(
+                [[1, 0, 1, 0, 1, 1],
+                 [1, 1, 1, 0, 0, 1],
+                 [0, 1, 0, 1, 1, 0],
+                 [1, 0, 1, 0, 0, 1]]
+            )
+
+        isplit = 2
+        sparsity = setup_sparsity(mask)
+        indeps, conns = setup_indeps(isplit, mask.shape[1], 'indeps', 'comp')
+
+        model.add_subsystem('indeps', indeps)
+        comp = model.add_subsystem('comp', SparseCompExplicit(sparsity, 'cs',
+                                                              isplit=isplit, osplit=2))
+        comp.declare_coloring('x*', method='cs', min_improve_pct=20)
+
+        for conn in conns:
+            model.connect(*conn)
+
+        prob.setup(check=False, mode='fwd')
+        prob.set_solver_print(level=0)
+        prob.run_model()
+
+        with assert_warning(UserWarning, 'SparseCompExplicit (comp): Coloring was deactivated.  Improvement of 16.7% was less than min allowed (20.0%).'):
+            comp._linearize()
+
+        jac = comp._jacobian._subjacs_info
+        _check_partial_matrix(comp, jac, sparsity, 'cs')
+
     @parameterized.expand(itertools.product(
         ['fd', 'cs'],
         ), name_func=_test_func_name
@@ -638,6 +673,56 @@ class TestColoring(unittest.TestCase):
         nruns = model._nruns - start_nruns
         self.assertEqual(nruns, 3)
 
+    @parameterized.expand(itertools.product(
+        [pyOptSparseDriver, ScipyOptimizeDriver],
+        ), name_func=_test_func_name
+    )
+    def test_simple_totals_min_improvement(self, optim):
+        prob = Problem(coloring_dir=self.tempdir)
+        model = prob.model = CounterGroup()
+        if optim is None:
+            raise unittest.SkipTest('requires pyoptsparse SLSQP.')
+        prob.driver = optim(optimizer='SLSQP')
+
+        prob.driver.declare_coloring()
+
+        mask = np.array(
+            [[1, 0, 1, 1, 1],
+             [1, 1, 0, 1, 1],
+             [0, 1, 1, 1, 1],
+             [1, 0, 1, 0, 0],
+             [0, 1, 1, 0, 1]]
+        )
+
+        isplit = 2
+        sparsity = setup_sparsity(mask)
+        indeps, conns = setup_indeps(isplit, mask.shape[1], 'indeps', 'comp')
+
+        model.add_subsystem('indeps', indeps)
+        comp = model.add_subsystem('comp', SparseCompExplicit(sparsity, 'cs', isplit=isplit, osplit=2))
+        model.connect('indeps.x0', 'comp.x0')
+        model.connect('indeps.x1', 'comp.x1')
+
+        model.comp.add_objective('y0', index=0)  # pyoptsparse SLSQP requires a scalar objective, so pick index 0
+        model.comp.add_constraint('y1', lower=[1., 2.])
+        model.add_design_var('indeps.x0', lower=np.ones(3), upper=np.ones(3)+.1)
+        model.add_design_var('indeps.x1', lower=np.ones(2), upper=np.ones(2)+.1)
+        model.approx_totals(method='cs')
+        model.declare_coloring(min_improve_pct=25.)
+        prob.setup(check=False, mode='fwd')
+        prob.set_solver_print(level=0)
+
+        with assert_warning(UserWarning, "CounterGroup (<model>): Coloring was deactivated.  Improvement of 20.0% was less than min allowed (25.0%)."):
+            prob.run_driver()  # need this to trigger the dynamic coloring
+
+        prob.driver._total_jac = None
+
+        start_nruns = model._nruns
+        derivs = prob.compute_totals()
+        nruns = model._nruns - start_nruns
+        _check_total_matrix(model, derivs, sparsity[[0,3,4],:], 'cs')
+        self.assertEqual(nruns, 5)
+
     @parameterized.expand(itertools.product(
         ['fd', 'cs'],
         ), name_func=_test_func_name
