@@ -4,12 +4,11 @@ import unittest
 
 import numpy as np
 
-from openmdao.api import Problem, Group, IndepVarComp, ExplicitComponent, \
-    ScipyOptimizeDriver, ImplicitComponent, LinearBlockGS
+import openmdao.api as om
 from openmdao.utils.assert_utils import assert_rel_error
 
 
-class QuadraticCompVectorized(ImplicitComponent):
+class QuadraticCompVectorized(om.ImplicitComponent):
     """
     A Simple Implicit Component representing a Quadratic Equation.
 
@@ -79,7 +78,7 @@ class QuadraticCompVectorized(ImplicitComponent):
             d_residuals['x'] = self.inv_jac * d_outputs['x']
 
 
-class QCVProblem(Problem):
+class QCVProblem(om.Problem):
     """
     A QuadraticCompVectorized problem with configurable component class.
     """
@@ -89,7 +88,7 @@ class QCVProblem(Problem):
 
         model = self.model
 
-        comp1 = model.add_subsystem('p', IndepVarComp())
+        comp1 = model.add_subsystem('p', om.IndepVarComp())
         comp1.add_output('a', np.array([1.0, 2.0, 3.0]))
         comp1.add_output('b', np.array([2.0, 3.0, 4.0]))
         comp1.add_output('c', np.array([-1.0, -2.0, -3.0]))
@@ -104,10 +103,10 @@ class QCVProblem(Problem):
         model.add_design_var('p.c', vectorize_derivs=True)
         model.add_constraint('comp.x', vectorize_derivs=True)
 
-        model.linear_solver = LinearBlockGS()
+        model.linear_solver = om.LinearBlockGS()
 
 
-class RectangleCompVectorized(ExplicitComponent):
+class RectangleCompVectorized(om.ExplicitComponent):
     """
     A simple Explicit Component that computes the area of a rectangle.
     """
@@ -283,7 +282,7 @@ def lagrange_matrices(x_disc, x_interp):
     return Li, Di
 
 
-class LGLFit(ExplicitComponent):
+class LGLFit(om.ExplicitComponent):
     """
     Given values at discretization nodes, provide interpolated values at midpoint nodes and
     an approximation of arclength.
@@ -316,7 +315,7 @@ class LGLFit(ExplicitComponent):
         outputs['yp_lgl'] = np.dot(self.D_lgl, inputs['y_lgl'])/np.pi
 
 
-class DefectComp(ExplicitComponent):
+class DefectComp(om.ExplicitComponent):
 
     def initialize(self):
         self.options.declare(name='num_nodes', types=int)
@@ -336,7 +335,7 @@ class DefectComp(ExplicitComponent):
         outputs['defect'] = inputs['y_truth'] - inputs['y_approx']
 
 
-class ArcLengthFunction(ExplicitComponent):
+class ArcLengthFunction(om.ExplicitComponent):
 
     def initialize(self):
         self.options.declare(name='num_nodes', types=int)
@@ -357,7 +356,7 @@ class ArcLengthFunction(ExplicitComponent):
         partials['f_arclength', 'yp_lgl'] = inputs['yp_lgl'] / np.sqrt(1 + inputs['yp_lgl']**2)
 
 
-class ArcLengthQuadrature(ExplicitComponent):
+class ArcLengthQuadrature(om.ExplicitComponent):
     """
     Computes the arclength of a polynomial segment whose values are given at the LGL nodes.
     """
@@ -380,7 +379,7 @@ class ArcLengthQuadrature(ExplicitComponent):
 
         da_df = np.atleast_2d(self.w_lgl*np.pi*self._mask)
 
-        self.declare_partials(of='arclength', wrt='f_arclength', dependent=True, val=da_df)
+        self.declare_partials(of='arclength', wrt='f_arclength', val=da_df)
 
     def compute(self, inputs, outputs):
         n = self.options['num_nodes']
@@ -391,7 +390,7 @@ class ArcLengthQuadrature(ExplicitComponent):
         outputs['arclength'] = outputs['arclength']*np.pi
 
 
-class Phase(Group):
+class Phase(om.Group):
 
     def initialize(self):
         self.options.declare('order', types=int, default=10)
@@ -401,16 +400,16 @@ class Phase(Group):
         n = order + 1
 
         # Step 1:  Make an indep var comp that provides the approximated values at the LGL nodes.
-        self.add_subsystem('y_lgl_ivc', IndepVarComp('y_lgl', val=np.zeros(n), desc='values at LGL nodes'),
+        self.add_subsystem('y_lgl_ivc', om.IndepVarComp('y_lgl', val=np.zeros(n), desc='values at LGL nodes'),
                            promotes_outputs=['y_lgl'])
 
         # Step 2:  Make an indep var comp that provides the 'truth' values at the midpoint nodes.
         x_lgl, _ = lgl(n)
         x_lgl = x_lgl * np.pi  # put x_lgl on [-pi, pi]
         x_mid = (x_lgl[1:] + x_lgl[:-1]) * 0.5  # midpoints on [-pi, pi]
-        self.add_subsystem('truth', IndepVarComp('y_mid',
-                                                 val=np.sin(x_mid),
-                                                 desc='truth values at midpoint nodes'))
+        self.add_subsystem('truth', om.IndepVarComp('y_mid',
+                                                    val=np.sin(x_mid),
+                                                    desc='truth values at midpoint nodes'))
 
         # Step 3: Make a polynomial fitting component
         self.add_subsystem('lgl_fit', LGLFit(num_nodes=n))
@@ -429,7 +428,7 @@ class Phase(Group):
         self.connect('arclength_func.f_arclength', 'arclength_quad.f_arclength')
 
 
-class Summer(ExplicitComponent):
+class Summer(om.ExplicitComponent):
 
     def initialize(self):
         self.options.declare('n_phases', types=int)
@@ -451,20 +450,20 @@ class Summer(ExplicitComponent):
 def simple_model(order, dvgroup='pardv', congroup='parc', vectorize=False):
     n = order + 1
 
-    p = Problem(model=Group())
+    p = om.Problem()
 
     # Step 1:  Make an indep var comp that provides the approximated values at the LGL nodes.
-    p.model.add_subsystem('y_lgl_ivc', IndepVarComp('y_lgl', val=np.zeros(n),
-                                                    desc='values at LGL nodes'),
+    p.model.add_subsystem('y_lgl_ivc', om.IndepVarComp('y_lgl', val=np.zeros(n),
+                                                       desc='values at LGL nodes'),
                           promotes_outputs=['y_lgl'])
 
     # Step 2:  Make an indep var comp that provides the 'truth' values at the midpoint nodes.
     x_lgl, _ = lgl(n)
     x_lgl = x_lgl * np.pi  # put x_lgl on [-pi, pi]
     x_mid = (x_lgl[1:] + x_lgl[:-1])/2.0  # midpoints on [-pi, pi]
-    p.model.add_subsystem('truth', IndepVarComp('y_mid',
-                                                val=np.sin(x_mid),
-                                                desc='truth values at midpoint nodes'))
+    p.model.add_subsystem('truth', om.IndepVarComp('y_mid',
+                                                   val=np.sin(x_mid),
+                                                   desc='truth values at midpoint nodes'))
 
     # Step 3: Make a polynomial fitting component
     p.model.add_subsystem('lgl_fit', LGLFit(num_nodes=n))
@@ -487,7 +486,7 @@ def simple_model(order, dvgroup='pardv', congroup='parc', vectorize=False):
     p.model.add_constraint('defect.defect', lower=-1e-6, upper=1e-6,
                            parallel_deriv_color=congroup, vectorize_derivs=vectorize)
     p.model.add_objective('arclength_quad.arclength')
-    p.driver = ScipyOptimizeDriver()
+    p.driver = om.ScipyOptimizeDriver()
     return p, np.sin(x_lgl)
 
 
@@ -497,7 +496,7 @@ def phase_model(order, nphases, dvgroup='pardv', congroup='parc', vectorize=Fals
 
     n = PHASE_ORDER + 1
 
-    p = Problem()
+    p = om.Problem()
 
     # Step 1:  Make an indep var comp that provides the approximated values at the LGL nodes.
     for i in range(N_PHASES):
@@ -513,7 +512,7 @@ def phase_model(order, nphases, dvgroup='pardv', congroup='parc', vectorize=Fals
     p.model.add_subsystem('sum', Summer(n_phases=N_PHASES))
 
     p.model.add_objective('sum.total_arc_length')
-    p.driver = ScipyOptimizeDriver()
+    p.driver = om.ScipyOptimizeDriver()
 
     x_lgl, _ = lgl(n)
     x_lgl = x_lgl * np.pi  # put x_lgl on [-pi, pi]
@@ -526,11 +525,11 @@ class MatMatTestCase(unittest.TestCase):
 
     def test_feature_vectorized_derivs(self):
         import numpy as np
-        from openmdao.api import ExplicitComponent, IndepVarComp, Problem, ScipyOptimizeDriver
+        import openmdao.api as om
 
         SIZE = 5
 
-        class ExpensiveAnalysis(ExplicitComponent):
+        class ExpensiveAnalysis(om.ExplicitComponent):
 
             def setup(self):
 
@@ -551,7 +550,7 @@ class MatMatTestCase(unittest.TestCase):
                 J['f', 'x'] = inputs['y']*inputs['x']**(inputs['y']-1)
                 J['f', 'y'] = (inputs['x']**inputs['y'])*np.log(inputs['x'])
 
-        class CheapConstraint(ExplicitComponent):
+        class CheapConstraint(om.ExplicitComponent):
 
             def setup(self):
 
@@ -571,9 +570,9 @@ class MatMatTestCase(unittest.TestCase):
 
                 J['g', 'y'] = 2*inputs['y']
 
-        p = Problem()
+        p = om.Problem()
 
-        dvs = p.model.add_subsystem('des_vars', IndepVarComp(), promotes=['*'])
+        dvs = p.model.add_subsystem('des_vars', om.IndepVarComp(), promotes=['*'])
         dvs.add_output('x', 2*np.ones(SIZE))
         dvs.add_output('y', 2*np.ones(SIZE))
 
@@ -589,7 +588,7 @@ class MatMatTestCase(unittest.TestCase):
 
         p.run_model()
 
-        p.driver = ScipyOptimizeDriver()
+        p.driver = om.ScipyOptimizeDriver()
         p.run_driver()
 
         assert_rel_error(self, p['x'], [0.10000691, 0.1, 0.1, 0.1, 0.1], 1e-5)
@@ -654,10 +653,10 @@ class MatMatTestCase(unittest.TestCase):
 
     def test_feature_declaration(self):
         # Tests the code that shows the signature for compute_multi_jacvec
-        prob = Problem()
+        prob = om.Problem()
         model = prob.model
 
-        comp1 = model.add_subsystem('p', IndepVarComp())
+        comp1 = model.add_subsystem('p', om.IndepVarComp())
         comp1.add_output('length', np.array([3.0, 4.0, 5.0]))
         comp1.add_output('width', np.array([1.0, 2.0, 3.0]))
 
@@ -785,7 +784,7 @@ class MatMatTestCase(unittest.TestCase):
                          "when it is read only.")
 
 
-class JacVec(ExplicitComponent):
+class JacVec(om.ExplicitComponent):
 
     def __init__(self, size):
         super(JacVec, self).__init__()
@@ -822,10 +821,10 @@ class MultiJacVec(JacVec):
 
 class ComputeMultiJacVecTestCase(unittest.TestCase):
     def setup_model(self, size, comp_class, vectorize, mode):
-        p = Problem()
+        p = om.Problem()
         model = p.model
-        model.add_subsystem('px', IndepVarComp('x', val=(np.arange(5, dtype=float) + 1.) * 3.0))
-        model.add_subsystem('py', IndepVarComp('y', val=(np.arange(5, dtype=float) + 1.) * 2.0))
+        model.add_subsystem('px', om.IndepVarComp('x', val=(np.arange(5, dtype=float) + 1.) * 3.0))
+        model.add_subsystem('py', om.IndepVarComp('y', val=(np.arange(5, dtype=float) + 1.) * 2.0))
         model.add_subsystem('comp', comp_class(size))
 
         model.connect('px.x', 'comp.x')
