@@ -56,26 +56,27 @@ _use_partial_sparsity = True
 
 # If True, ignore use_fixed_coloring if the coloring passed to it is _STD_COLORING_FNAME.
 # This is used when the 'openmdao partial_coloring' or 'openmdao total_coloring' commands
-# are running, because the intent there is to generate new coloring files.
+# are running, because the intent there is to generate new coloring files regardless of
+# whether use_fixed_coloring was called.
 _force_dyn_coloring = False
 
 # used as an indicator that we should automatically name coloring file based on class module
 # path or system pathname
 _STD_COLORING_FNAME = object()
 
-# used to indicate that we should dynamically generate a coloring
-_DYN_COLORING = object()
 
 # default values related to the computation of a sparsity matrix
 _DEF_COMP_SPARSITY_ARGS = {
-    'tol': 1e-15,
-    'orders': 15,
-    'num_full_jacs': 3,
-    'perturb_size': 1e-9,
-    'show_summary': True,
-    'show_sparsity': False,
+    'tol': 1e-25,     # use this tolerance to determine what's a zero when determining sparsity
+    'orders': None,   # num orders += around 'tol' for the tolerance sweep when determining sparsity
+    'num_full_jacs': 3,      # number of full jacobians to generate before computing sparsity
+    'perturb_size': 1e-9,    # size of input/output perturbation during generation of sparsity
+    'min_improve_pct': 5.,   # don't use coloring unless at least 5% decrease in number of solves
+    'show_summary': True,    # if True, print a short summary of the coloring
+    'show_sparsity': False,  # if True, show a plot of the sparsity
 }
 
+
 # numpy versions before 1.12 don't use the 'axis' arg passed to count_nonzero and always
 # return an int instead of an array of ints, so create our own function for those versions.
 if LooseVersion(np.__version__) >= LooseVersion("1.12"):
@@ -263,11 +264,11 @@ class Coloring(object):
         rev_size = self._shape[0] if self._shape else -1  # nrows
         fwd_size = self._shape[1] if self._shape else -1  # ncols
 
-        tot_colors = self.total_solves()
+        tot_solves = self.total_solves()
 
         fwd_solves = rev_solves = 0
-        if tot_colors == 0:  # no coloring found
-            tot_colors = tot_size = min([rev_size, fwd_size])
+        if tot_solves == 0:  # no coloring found
+            tot_solves = tot_size = min([rev_size, fwd_size])
             pct = 0.
         else:
             fwd_lists = self._fwd[0] if self._fwd else []
@@ -289,12 +290,12 @@ class Coloring(object):
             if tot_size <= 0:
                 pct = 0.
             else:
-                pct = ((tot_size - tot_colors) / tot_size * 100)
+                pct = ((tot_size - tot_solves) / tot_size * 100)
 
         if tot_size < 0:
             tot_size = '?'
 
-        return tot_size, tot_colors, fwd_solves, rev_solves, pct
+        return tot_size, tot_solves, fwd_solves, rev_solves, pct
 
     def total_solves(self, do_fwd=True, do_rev=True):
         """
@@ -373,17 +374,12 @@ class Coloring(object):
         driver : Driver
             Current driver object.
         """
-        ofs = driver._get_ordered_nl_responses()
-        of_sizes = _get_response_sizes(driver, ofs)
-        wrts = list(driver._designvars)
-        wrt_sizes = _get_desvar_sizes(driver, wrts)
-        if (self._row_var_sizes != of_sizes or self._row_vars != ofs
-                or self._col_var_sizes != wrt_sizes or self._col_vars != wrts):
-            raise RuntimeError("Current total coloring configuration does not match the "
-                               "configuration of the current driver. Make sure you don't have "
-                               "different problems that have the same coloring directory.  Set "
-                               "the coloring directory by setting the value of "
-                               "`problem.options['coloring_dir']`.")
+        of_names = driver._get_ordered_nl_responses()
+        of_sizes = _get_response_sizes(driver, of_names)
+        wrt_names = list(driver._designvars)
+        wrt_sizes = _get_desvar_sizes(driver, wrt_names)
+
+        self._config_check_msgs(of_names, of_sizes, wrt_names, wrt_sizes, driver)
 
     def _check_config_partial(self, system):
         """
@@ -395,27 +391,75 @@ class Coloring(object):
             System being colored.
         """
         # check the contents (vars and sizes) of the input and output vectors of system
+        info = {'coloring': None, 'wrt_patterns': self._meta['wrt_patterns']}
+        system._update_wrt_matches(info)
         if system.pathname:
             wrt_matches = ['.'.join((system.pathname, n))
-                           for n in self._meta['wrt_matches_prom']]
+                           for n in info['wrt_matches_prom']]
             # for partial and semi-total derivs, convert to promoted names
             ordered_of_info = system._jac_var_info_abs2prom(system._jacobian_of_iter())
             ordered_wrt_info = \
                 system._jac_var_info_abs2prom(system._jacobian_wrt_iter(wrt_matches))
         else:
             ordered_of_info = list(system._jacobian_of_iter())
-            ordered_wrt_info = list(system._jacobian_wrt_iter(self._meta['wrt_matches']))
+            ordered_wrt_info = list(system._jacobian_wrt_iter(info['wrt_matches']))
 
-        ordered_of_names = [t[0] for t in ordered_of_info]
-        ordered_wrt_names = [t[0] for t in ordered_wrt_info]
+        of_names = [t[0] for t in ordered_of_info]
+        wrt_names = [t[0] for t in ordered_wrt_info]
 
-        if (ordered_of_names != self._row_vars or ordered_wrt_names != self._col_vars):
-            # TODO: add comparison of sizes
-            raise RuntimeError("%s: Current coloring configuration does not match the "
-                               "configuration of the current driver. Make sure you don't have "
-                               "different problems that have the same coloring directory.  Set "
-                               "the coloring directory by setting the value of "
-                               "`problem.options['coloring_dir']`." % system.pathname)
+        of_sizes = [t[2] - t[1] for t in ordered_of_info]
+        wrt_sizes = [t[2] - t[1] for t in ordered_wrt_info]
+
+        self._config_check_msgs(of_names, of_sizes, wrt_names, wrt_sizes, system)
+
+    def _config_check_msgs(self, of_names, of_sizes, wrt_names, wrt_sizes, obj):
+        msg_suffix = ("Make sure you don't have different problems that have the same coloring "
+                      "directory. Set the coloring directory by setting the value of "
+                      "problem.options['coloring_dir'].")
+
+        msg = ["%s: Current coloring configuration does not match the "
+               "configuration of the current model." % obj.msginfo]
+
+        if of_names != self._row_vars:
+            of_diff = set(of_names) - set(self._row_vars)
+            if of_diff:
+                msg.append('   The following row vars were added: %s.' % sorted(of_diff))
+            else:
+                of_diff = set(self._row_vars) - set(of_names)
+                if of_diff:
+                    msg.append('   The following row vars were removed: %s.' % sorted(of_diff))
+                else:
+                    msg.append('   The row vars have changed order.')
+
+        if wrt_names != self._col_vars:
+            wrt_diff = set(wrt_names) - set(self._col_vars)
+            if wrt_diff:
+                msg.append('   The following column vars were added: %s.' % sorted(wrt_diff))
+            else:
+                wrt_diff = set(self._col_vars) - set(wrt_names)
+                if wrt_diff:
+                    msg.append('   The following column vars were removed: %s.' % sorted(wrt_diff))
+                else:
+                    msg.append('   The column vars have changed order.')
+
+        # check sizes
+        changed_sizes = []
+        if of_names == self._row_vars:
+            for i, (my_sz, sz) in enumerate(zip(self._row_var_sizes, of_sizes)):
+                if my_sz != sz:
+                    changed_sizes.append(of_names[i])
+
+        if wrt_names == self._col_vars:
+            for i, (my_sz, sz) in enumerate(zip(self._col_var_sizes, wrt_sizes)):
+                if my_sz != sz:
+                    changed_sizes.append(wrt_names[i])
+
+        if changed_sizes:
+            msg.append('   The following variables have changed sizes: %s.' % sorted(changed_sizes))
+
+        if len(msg) > 1:
+            msg.append(msg_suffix)
+            raise RuntimeError('\n'.join(msg))
 
     def __repr__(self):
         """
@@ -468,8 +512,8 @@ class Coloring(object):
         good_tol = meta.get('good_tol')
         if good_tol is not None:
             print("\nSparsity computed using tolerance: %g" % meta['good_tol'])
-            print("Most common number of zero entries (%d of %d) repeated %d times out of %d "
-                  "tolerances tested.\n" % (meta['zero_entries'], meta['J_size'],
+            print("Most common number of nonzero entries (%d of %d) repeated %d times out of %d "
+                  "tolerances tested.\n" % (meta['J_size'] - meta['zero_entries'], meta['J_size'],
                                             meta['nz_matches'], meta['n_tested']))
 
         sparsity_time = meta.get('sparsity_time')
@@ -1173,7 +1217,7 @@ def MNCO_bidir(J):
     return coloring
 
 
-def _tol_sweep(arr, tol=1e-15, orders=20):
+def _tol_sweep(arr, tol=_DEF_COMP_SPARSITY_ARGS['tol'], orders=_DEF_COMP_SPARSITY_ARGS['orders']):
     """
     Find best tolerance 'around' tol to choose nonzero values of arr.
 
@@ -1212,7 +1256,7 @@ def _tol_sweep(arr, tol=1e-15, orders=20):
                 else:
                     nzeros.append(([itol], len(rows)))
                 n_tested += 1
-            itol /= 10.
+            itol *= .1
 
         # pick lowest tolerance corresponding to the most repeated number of 'zero' entries
         sorted_items = sorted(nzeros, key=lambda x: len(x[0]), reverse=True)
@@ -1258,6 +1302,8 @@ def _compute_total_coloring_context(top):
     top : System
         Top of the system hierarchy where coloring will be done.
     """
+    np.random.seed(41)  # set seed for consistency
+
     for system in top.system_iter(recurse=True, include_self=True):
         if system.matrix_free:
             raise RuntimeError("%s: simultaneous coloring does not currently work with matrix free "
@@ -1334,7 +1380,7 @@ def _get_bool_total_jac(prob, num_full_jacs=_DEF_COMP_SPARSITY_ARGS['num_full_ja
                 fullJ += np.abs(J)
         elapsed = time.time() - start_time
 
-    fullJ *= (1.0 / num_full_jacs)
+    fullJ *= (1.0 / np.max(fullJ))
 
     info = _tol_sweep(fullJ, tol, orders)
     info['num_full_jacs'] = num_full_jacs
@@ -1522,17 +1568,25 @@ def _compute_coloring(J, mode):
         See Coloring class docstring.
     """
     start_time = time.time()
+    nrows, ncols = J.shape
+    best_nocolor = min(nrows, ncols)  # lowest number of solves possible if we don't use coloring
 
     if mode == 'auto':  # use bidirectional coloring
-        return MNCO_bidir(J)
+        coloring = MNCO_bidir(J)
+        if coloring.total_solves() < best_nocolor:
+            return coloring
+        elif ncols <= nrows:
+            mode = 'fwd'
+        else:
+            mode == 'rev'
 
     rev = mode == 'rev'
-    nrows, ncols = J.shape
 
     coloring = Coloring(sparsity=J)
 
     if rev:
         J = J.T
+
     col_groups = _split_groups(_get_full_disjoint_cols(J))
 
     full_slice = slice(None)
@@ -1541,10 +1595,10 @@ def _compute_coloring(J, mode):
         for col in lst:
             col2rows[col] = np.nonzero(J[:, col])[0]
 
-    if mode == 'fwd':
-        coloring._fwd = (col_groups, col2rows)
-    else:
+    if rev:
         coloring._rev = (col_groups, col2rows)
+    else:  # fwd
+        coloring._fwd = (col_groups, col2rows)
 
     coloring._meta['coloring_time'] = time.time() - start_time
 
@@ -1612,7 +1666,10 @@ def compute_total_coloring(problem, mode=None,
                                           "from nonlinear ones.")
             _initialize_model_approx(model, driver, ofs, wrts)
             if model._coloring_info['coloring'] is None:
-                model.declare_coloring(method=list(model._approx_schemes)[0])
+                kwargs = {n: v for n, v in model._coloring_info.items()
+                          if n in _DEF_COMP_SPARSITY_ARGS and v is not None}
+                kwargs['method'] = list(model._approx_schemes)[0]
+                model.declare_coloring(**kwargs)
             if run_model:
                 problem.run_model()
             coloring = model._compute_approx_coloring(wrt_patterns='*',
@@ -1624,21 +1681,22 @@ def compute_total_coloring(problem, mode=None,
                                                    orders=orders, setup=setup,
                                                    run_model=run_model)
             coloring = _compute_coloring(J, mode)
-            coloring._row_vars = ofs
-            coloring._row_var_sizes = of_sizes
-            coloring._col_vars = wrts
-            coloring._col_var_sizes = wrt_sizes
+            if coloring is not None:
+                coloring._row_vars = ofs
+                coloring._row_var_sizes = of_sizes
+                coloring._col_vars = wrts
+                coloring._col_var_sizes = wrt_sizes
 
-            # save metadata we used to create the coloring
-            coloring._meta.update(sparsity_info)
+                # save metadata we used to create the coloring
+                coloring._meta.update(sparsity_info)
 
-            driver._total_jac = None
+                driver._total_jac = None
 
-            system = problem.model
-            if fname is not None:
-                if ((system._full_comm is not None and system._full_comm.rank == 0) or
-                        (system._full_comm is None and system.comm.rank == 0)):
-                    coloring.save(fname)
+                system = problem.model
+                if fname is not None:
+                    if ((system._full_comm is not None and system._full_comm.rank == 0) or
+                            (system._full_comm is None and system.comm.rank == 0)):
+                        coloring.save(fname)
 
     elif bool_jac is not None:
         J = bool_jac
@@ -1646,7 +1704,7 @@ def compute_total_coloring(problem, mode=None,
             mode = 'auto'
         driver = None
         coloring = _compute_coloring(J, mode)
-        if fname is not None:
+        if coloring is not None and fname is not None:
             coloring.save(fname)
     else:
         raise RuntimeError("You must supply either problem or bool_jac to "
@@ -1718,14 +1776,15 @@ def dynamic_total_coloring(driver, run_model=True, fname=None):
     coloring = compute_total_coloring(problem, num_full_jacs=num_full_jacs, tol=tol, orders=orders,
                                       setup=False, run_model=run_model, fname=fname)
 
-    if driver._coloring_info['show_sparsity']:
-        coloring.display_txt()
-    if driver._coloring_info['show_summary']:
-        coloring.summary()
+    if coloring is not None:
+        if driver._coloring_info['show_sparsity']:
+            coloring.display_txt()
+        if driver._coloring_info['show_summary']:
+            coloring.summary()
 
-    driver._coloring_info['coloring'] = coloring
-    driver._setup_simul_coloring()
-    driver._setup_tot_jac_sparsity()
+        driver._coloring_info['coloring'] = coloring
+        driver._setup_simul_coloring()
+        driver._setup_tot_jac_sparsity()
 
     return coloring
 
@@ -1804,11 +1863,12 @@ def _total_coloring_cmd(options):
                                                   orders=options.orders,
                                                   setup=False, run_model=True, fname=outfile)
 
-            if options.show_sparsity_text:
-                coloring.display_txt()
-            if options.show_sparsity:
-                coloring.display()
-            coloring.summary()
+            if coloring is not None:
+                if options.show_sparsity_text:
+                    coloring.display_txt()
+                if options.show_sparsity:
+                    coloring.display()
+                coloring.summary()
         else:
             print("Derivatives are turned off.  Cannot compute simul coloring.")
         exit()
@@ -1953,7 +2013,8 @@ def _partial_coloring_cmd(options):
                                     print("The following error occurred while attempting to "
                                           "compute coloring for %s:\n %s" % (s.pathname, tb))
                                 else:
-                                    _show(s, options, coloring)
+                                    if coloring is not None:
+                                        _show(s, options, coloring)
                                 if options.norecurse:
                                     break
                     else:
@@ -1967,9 +2028,10 @@ def _partial_coloring_cmd(options):
                         print("No coloring found.")
                     else:
                         for c in colorings:
-                            path = c._meta['pathname']
-                            s = prob.model._get_subsystem(path) if path else prob.model
-                            _show(s, options, c)
+                            if c is not None:
+                                path = c._meta['pathname']
+                                s = prob.model._get_subsystem(path) if path else prob.model
+                                _show(s, options, c)
         else:
             print("Derivatives are turned off.  Cannot compute simul coloring.")
         exit()
