@@ -6,11 +6,19 @@ from __future__ import print_function
 import sys
 import os
 import argparse
+from itertools import chain
 from six import iteritems
 
 from openmdao.core.problem import Problem
-from openmdao.devtools.problem_viewer.problem_viewer import view_model
-from openmdao.devtools.viewconns import view_connections
+from openmdao.visualization.n2_viewer.n2_viewer import n2
+from openmdao.visualization.connection_viewer.viewconns import view_connections
+from openmdao.components.meta_model_unstructured_comp import MetaModelUnStructuredComp
+from openmdao.components.meta_model_structured_comp import MetaModelStructuredComp
+try:
+    import bokeh
+    from openmdao.visualization.meta_model_viewer.meta_model_visualization import view_metamodel
+except ImportError:
+    bokeh = None
 from openmdao.devtools.debug import config_summary, tree, dump_dist_idxs
 from openmdao.devtools.itrace import _itrace_exec, _itrace_setup_parser
 from openmdao.devtools.iprofile_app.iprofile_app import _iprof_exec, _iprof_setup_parser
@@ -18,7 +26,7 @@ from openmdao.devtools.iprofile import _iprof_totals_exec, _iprof_totals_setup_p
 from openmdao.devtools.iprof_mem import _mem_prof_exec, _mem_prof_setup_parser, \
     _mempost_exec, _mempost_setup_parser
 from openmdao.devtools.iprof_utils import _Options
-from openmdao.devtools.xdsm_viewer.xdsm_writer import write_xdsm, \
+from openmdao.visualization.xdsm_viewer.xdsm_writer import write_xdsm, \
     _DEFAULT_BOX_STACKING, _DEFAULT_BOX_WIDTH, _MAX_BOX_LINES, _DEFAULT_OUTPUT_SIDE, _CHAR_SUBS
 from openmdao.error_checking.check_config import _check_config_cmd, _check_config_setup_parser
 from openmdao.utils.mpi import MPI
@@ -27,11 +35,13 @@ from openmdao.utils.code_utils import _calltree_setup_parser, _calltree_exec
 from openmdao.utils.coloring import _total_coloring_setup_parser, _total_coloring_cmd, \
     _sparsity_setup_parser, _sparsity_cmd, _partial_coloring_setup_parser, _partial_coloring_cmd, \
     _view_coloring_setup_parser, _view_coloring_exec
+from openmdao.utils.scaffold import _scaffold_setup_parser, _scaffold_exec
+from openmdao.utils.general_utils import warn_deprecation
 
 
-def _view_model_setup_parser(parser):
+def _n2_setup_parser(parser):
     """
-    Set up the openmdao subparser for the 'openmdao view_model' command.
+    Set up the openmdao subparser for the 'openmdao n2' command.
 
     Parameters
     ----------
@@ -52,9 +62,9 @@ def _view_model_setup_parser(parser):
                         help="use declare partial info for internal connectivity.")
 
 
-def _view_model_cmd(options):
+def _n2_cmd(options):
     """
-    Process command line args and call view_model on the specified file.
+    Process command line args and call n2 on the specified file.
 
     Parameters
     ----------
@@ -66,25 +76,23 @@ def _view_model_cmd(options):
     if filename.endswith('.py'):
         # the file is a python script, run as a post_setup hook
         def _viewmod(prob):
-            view_model(prob, outfile=options.outfile,
-                       show_browser=not options.no_browser,
-                       title=options.title,
-                       embeddable=options.embeddable,
-                       use_declare_partial_info=options.use_declare_partial_info
-                       )
+            n2(prob, outfile=options.outfile, show_browser=not options.no_browser,
+               title=options.title, embeddable=options.embeddable,
+               use_declare_partial_info=options.use_declare_partial_info)
             exit()  # could make this command line selectable later
 
         options.func = lambda options: _viewmod
-
         _post_setup_exec(options)
     else:
         # assume the file is a recording, run standalone
-        view_model(filename, outfile=options.outfile,
-                   title=options.title,
-                   show_browser=not options.no_browser,
-                   embeddable=options.embeddable,
-                   use_declare_partial_info=options.use_declare_partial_info
-                   )
+        n2(filename, outfile=options.outfile, title=options.title,
+           show_browser=not options.no_browser, embeddable=options.embeddable,
+           use_declare_partial_info=options.use_declare_partial_info)
+
+
+def _view_model_cmd(options):
+    warn_deprecation("The 'view_model' command has been deprecated. Use 'n2' instead.")
+    _n2_cmd(options)
 
 
 def _xdsm_setup_parser(parser):
@@ -138,8 +146,10 @@ def _xdsm_setup_parser(parser):
                              'dictionary with component types as keys. Component type key can be '
                              '"optimization", "doe" or "default".')
     parser.add_argument('--legend', action='store_true', dest='legend',
-                        help='Position of the outputs on the diagram. Left or right, or a '
-                             'Show legend.')
+                        help='If True, show legend.')
+    parser.add_argument('--class_names', action='store_true', dest='class_names',
+                        help='If true, appends class name of the groups/components to the '
+                             'component blocks of the diagram..')
 
 
 def _xdsm_cmd(options):
@@ -171,6 +181,7 @@ def _xdsm_cmd(options):
                        add_process_conns=not options.no_process_conns,
                        output_side=options.output_side,
                        legend=options.legend,
+                       class_names=options.class_names,
                        **kwargs)
             exit()
 
@@ -201,6 +212,8 @@ def _view_connections_setup_parser(parser):
     parser.add_argument('file', nargs=1, help='Python file containing the model.')
     parser.add_argument('-o', default='connections.html', action='store', dest='outfile',
                         help='html output file.')
+    parser.add_argument('-t', '--title', action='store', dest='title',
+                        help='title of web page.')
     parser.add_argument('--no_browser', action='store_true', dest='no_browser',
                         help="don't display in a browser.")
 
@@ -220,11 +233,95 @@ def _view_connections_cmd(options):
         The post-setup hook function.
     """
     def _viewconns(prob):
-        view_connections(prob, outfile=options.outfile, show_browser=not options.no_browser)
+        if options.title:
+            title = options.title
+        else:
+            title = "Connections for %s" % os.path.basename(options.file[0])
+        view_connections(prob, outfile=options.outfile, show_browser=not options.no_browser,
+                         title=title)
         exit()
     return _viewconns
 
 
+def _meta_model_parser(parser):
+    """
+    Set up the openmdao subparser for the 'openmdao meta_model' command.
+
+    Parameters
+    ----------
+    parser : argparse subparser
+        The parser we're adding options to.
+    """
+    parser.add_argument('file', nargs=1, help='Python file containing the model.')
+    parser.add_argument('-m', '--metamodel_pathname', action='store', dest='pathname',
+                        help='pathname of the metamodel component.')
+    parser.add_argument('-p', '--port_number', default=5007, action='store', dest='port_number',
+                        help='Port number to open viewer')
+
+
+def _meta_model_cmd(options):
+    """
+    Return the post_setup hook function for 'openmdao meta_model'.
+
+    Parameters
+    ----------
+    options : argparse Namespace
+        Command line options.
+
+    Returns
+    -------
+    function
+        The post-setup hook function.
+    """
+    def _view_metamodel(prob):
+        if bokeh is None:
+            print("bokeh must be installed to view a MetaModel.  Use the command:\n",
+                  "    pip install bokeh")
+            exit()
+
+        Problem._post_setup_func = None
+
+        mm_types = (MetaModelStructuredComp, MetaModelUnStructuredComp)
+
+        pathname = options.pathname
+        port_number = options.port_number
+
+        if pathname:
+            comp = prob.model._get_subsystem(pathname)
+            if comp and isinstance(comp, mm_types):
+                view_metamodel(comp, port_number)
+                exit()
+        else:
+            comp = None
+
+        metamodels = {mm.pathname: mm for
+                      mm in prob.model.system_iter(include_self=True, typ=mm_types)}
+
+        mm_names = list(metamodels.keys())
+        mm_count = len(mm_names)
+
+        if mm_count == 0:
+            print("No Metamodel components found in model.")
+
+        elif mm_count == 1 and not pathname:
+            comp = metamodels[mm_names[0]]
+            view_metamodel(comp, port_number)
+            exit()
+
+        else:
+            try_str = "Try one of the following: {}.".format(mm_names)
+
+            if not pathname:
+                print("\nMetamodel not specified. {}".format(try_str))
+            elif not comp:
+                print("\nMetamodel '{}' not found.\n {}".format(pathname, try_str))
+            else:
+                print("\n'{}' is not a Metamodel.\n {}".format(pathname, try_str))
+        exit()
+
+    return _view_metamodel
+
+
 def _config_summary_setup_parser(parser):
     """
     Set up the openmdao subparser for the 'openmdao summary' command.
@@ -474,31 +571,50 @@ def _post_setup_exec(options):
 
 # All post-setup functions go here.
 # this dict should contain names mapped to tuples of the form:
-#   (setup_parser_func, func)
+#   (setup_parser_func, func, description)
 _post_setup_map = {
-    'view_connections': (_view_connections_setup_parser, _view_connections_cmd),
-    'summary': (_config_summary_setup_parser, _config_summary_cmd),
-    'tree': (_tree_setup_parser, _tree_cmd),
-    'dump_idxs': (_dump_dist_idxs_setup_parser, _dump_dist_idxs_cmd),
-    'total_coloring': (_total_coloring_setup_parser, _total_coloring_cmd),
-    'partial_coloring': (_partial_coloring_setup_parser, _partial_coloring_cmd),
-    'total_sparsity': (_sparsity_setup_parser, _sparsity_cmd),
-    'cite': (_cite_setup_parser, _cite_cmd),
-    'check': (_check_config_setup_parser, _check_config_cmd),
+    'view_connections': (_view_connections_setup_parser, _view_connections_cmd,
+                         'Connection viewer showing values and source/target units.'),
+    'summary': (_config_summary_setup_parser, _config_summary_cmd,
+                'Print a short top-level summary of the problem.'),
+    'tree': (_tree_setup_parser, _tree_cmd, 'Print the system tree.'),
+    'dump_idxs': (_dump_dist_idxs_setup_parser, _dump_dist_idxs_cmd,
+                  'Show distributed index information.'),
+    'total_coloring': (_total_coloring_setup_parser, _total_coloring_cmd,
+                       'Compute a coloring for the total jacobian.'),
+    'partial_coloring': (_partial_coloring_setup_parser, _partial_coloring_cmd,
+                         'Compute coloring(s) for specified partial jacobians.'),
+    'total_sparsity': (_sparsity_setup_parser, _sparsity_cmd,
+                       'Compute the sparsity pattern of the total jacobian.'),
+    'cite': (_cite_setup_parser, _cite_cmd,
+             'Print citations referenced by problem'),
+    'check': (_check_config_setup_parser, _check_config_cmd,
+              'Perform a number of configuration checks on the problem.'),
+    'view_mm': (_meta_model_parser, _meta_model_cmd, "Meta Model Viewer.")
 }
 
 
 # Other non-post-setup functions go here
 _non_post_setup_map = {
-    'view_model': (_view_model_setup_parser, _view_model_cmd),
-    'trace': (_itrace_setup_parser, _itrace_exec),
-    'call_tree': (_calltree_setup_parser, _calltree_exec),
-    'iprof': (_iprof_setup_parser, _iprof_exec),
-    'iprof_totals': (_iprof_totals_setup_parser, _iprof_totals_exec),
-    'mem': (_mem_prof_setup_parser, _mem_prof_exec),
-    'mempost': (_mempost_setup_parser, _mempost_exec),
-    'view_coloring': (_view_coloring_setup_parser, _view_coloring_exec),
-    'xdsm': (_xdsm_setup_parser, _xdsm_cmd),
+    'n2': (_n2_setup_parser, _n2_cmd, 'Display an interactive N2 diagram of the problem.'),
+    'view_model': (_n2_setup_parser, _view_model_cmd,
+                   'Display an interactive N2 diagram of the problem. '
+                   '(Deprecated, please use n2 instead.)'),
+    'trace': (_itrace_setup_parser, _itrace_exec, 'Dump trace output.'),
+    'call_tree': (_calltree_setup_parser, _calltree_exec,
+                  "Display the call tree for the specified class method and all 'self' class "
+                  "methods it calls."),
+    'iprof': (_iprof_setup_parser, _iprof_exec,
+              'Profiling of calls to particular object instances.'),
+    'iprof_totals': (_iprof_totals_setup_parser, _iprof_totals_exec,
+                     'Total timings of calls to particular object instances.'),
+    'mem': (_mem_prof_setup_parser, _mem_prof_exec, 'Memory profiler.'),
+    'mempost': (_mempost_setup_parser, _mempost_exec, 'Post-processor for memory profile output.'),
+    'view_coloring': (_view_coloring_setup_parser, _view_coloring_exec,
+                      'Colored jacobian viewer.'),
+    'xdsm': (_xdsm_setup_parser, _xdsm_cmd, 'XDSM viewer.'),
+    'scaffold': (_scaffold_setup_parser, _scaffold_exec,
+                 'Generate a simple scaffold for a component.'),
 }
 
 
@@ -506,18 +622,18 @@ def openmdao_cmd():
     """
     Wrap a number of Problem viewing/debugging command line functions.
     """
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(description='OpenMDAO Command Line Tools',
+                                     epilog='Use -h after any sub-command for sub-command help.')
 
-    subs = parser.add_subparsers(help='sub-command help')
-    for p, (parser_setup_func, cmd) in iteritems(_post_setup_map):
-        subp = subs.add_parser(p)
+    subs = parser.add_subparsers(title='Tools', metavar='')
+    for p, (parser_setup_func, cmd, help_str) in sorted(chain(_post_setup_map.items(),
+                                                              _non_post_setup_map.items())):
+        subp = subs.add_parser(p, help=help_str)
         parser_setup_func(subp)
-        subp.set_defaults(func=cmd, executor=_post_setup_exec)
-
-    for p, (parser_setup_func, cmd) in iteritems(_non_post_setup_map):
-        subp = subs.add_parser(p)
-        parser_setup_func(subp)
-        subp.set_defaults(executor=cmd)
+        if p in _post_setup_map:
+            subp.set_defaults(func=cmd, executor=_post_setup_exec)
+        else:
+            subp.set_defaults(executor=cmd)
 
     # handle case where someone just runs `openmdao <script>`
     args = [a for a in sys.argv[1:] if not a.startswith('-')]
